# -*- coding: utf-8 -*-
"""
envassume usage
"""

from __future__ import print_function
import sys


def print_help(short = False):
    print('usage: envassume [-h] [-i EXTERNAL_ID] [ARN] command [argument [argument ...]]', file = sys.stderr)
    if not short:
        print('''
optional arguments:
    -h, --help              show this help message and exit
    -i, --id EXTERNAL_ID    external id
    ARN                     AWS role ARN to assume (required if not set by environment variable)

environment variables:
    environment must contain valid AWS API credentials

    AWS_ASSUME_ROLE=ARN
        no options can be present before the command if this is defined

    AWS_ASSUME_ID=EXTERNAL_ID
''', file = sys.stderr)


def print_error(message):
    print_help(short = True)
    print('error: {}'.format(message), file = sys.stderr)



# -*- coding: utf-8 -*-
"""
envassume arguments
"""

from __future__ import print_function
import os
from attr import attrs, attrib
from .exceptions import (
    EnvAssumeHelpException,
    EnvAssumeArgumentException,
    EnvAssumeMissingArnException,
    EnvAssumeMissingCommandException,
)


@attrs
class Env(object):
    name = attrib()
    var = attrib()


ENV_LIST = (
    Env('arn', 'AWS_ASSUME_ROLE'),
    Env('external_id', 'AWS_ASSUME_ID'),
)


@attrs
class Option(object):
    name = attrib()
    arg_short = attrib()
    arg = attrib()


OPTION_LIST = (
    Option('external_id', '-i', '--id'),
)


def _pop_argument(arg_list):
    try:
        return arg_list.pop(0)

    except IndexError:
        return None


def _match_option(arg, arg_list):
    for option in OPTION_LIST:
        if arg == '-h' or arg == '--help':
            raise EnvAssumeHelpException

        elif arg == option.arg_short:
            return option.name, _pop_argument(arg_list)

        elif arg == option.arg:
            return option.name, _pop_argument(arg_list)

        elif arg.startswith(option.arg):
            arg_split = arg.split('=', 1)
            if arg_split[0] == option.arg and len(arg_split) == 2:
                return option.name, arg_split[1]

            raise EnvAssumeArgumentException('Invalid argument: {}'.format(arg))

        elif arg == '--':
            raise EnvAssumeMissingArnException('ARN not supplied')

    raise EnvAssumeArgumentException('Unknown argument: {}'.format(arg))


def parse_arguments(arg_list):
    # skip script name
    _pop_argument(arg_list)

    result = {}

    for env in ENV_LIST:
        env_val = os.environ.get(env.var)
        if env_val:
            result[env.name] = env_val

    if not result.get('arn'):
        while True:
            arg = _pop_argument(arg_list)

            if arg and arg.startswith('-'):
                option_name, option_value = _match_option(arg, arg_list)
                result[option_name] = option_value

            elif not arg:
                raise EnvAssumeMissingArnException('ARN not supplied')

            else:
                result['arn'] = arg
                break

    # skip optional separator
    if arg_list and arg_list[0] == '--':
        _pop_argument(arg_list)

    if not arg_list:
        raise EnvAssumeMissingCommandException('Command not supplied')

    result['command'] = arg_list

    return result



# -*- coding: utf-8 -*-
"""
envassume
"""

from __future__ import print_function
import sys
import os
from socket import gethostname
import boto3
from botocore.exceptions import BotoCoreError
from .exceptions import EnvAssumeException, EnvAssumeHelpException
from .arguments import parse_arguments
from .usage import print_help, print_error


def assume_role(role_arn, external_id = None, session_name = None):
    if not external_id:
        external_id = os.environ.get('AWS_EXTERNAL_ID')

    if not session_name:
        session_name = 'envassume-' + gethostname()

    request = {
        'RoleArn': role_arn,
        'RoleSessionName': session_name,
    }

    if external_id:
        request['ExternalId'] = external_id

    boto3_session = boto3.Session()
    sts_client = boto3_session.client('sts')
    try:
        response = sts_client.assume_role(**request)

    except BotoCoreError as e:
        raise EnvAssumeException(e.message)

    return response.get('Credentials') or {}


def update_env(credentials_lookup):
    credentials_list = (
        ('AccessKeyId', 'AWS_ACCESS_KEY_ID'),
        ('SecretAccessKey', 'AWS_SECRET_ACCESS_KEY'),
        ('SessionToken', 'AWS_SESSION_TOKEN'),
    )
    for credential_key, env_var_name in credentials_list:
        os.environ[env_var_name] = credentials_lookup[credential_key]


def exec_command(arg_list):
    try:
        os.execvpe(arg_list[0], arg_list, os.environ)

    except OSError as e:
        raise EnvAssumeException('Unable to run command {}: {}'.format(arg_list[0], e))

    raise EnvAssumeException('Unable to run command {}'.format(arg_list[0]))


def envassume():
    arg_lookup = parse_arguments(sys.argv)

    credentials_lookup = assume_role(arg_lookup.get('arn'), arg_lookup.get('external_id'))

    update_env(credentials_lookup)

    exec_command(arg_lookup['command'])


def run():
    try:
        envassume()

    except EnvAssumeHelpException:
        print_help()

    except EnvAssumeException as e:
        print_error(e)
        sys.exit(1)

    except KeyboardInterrupt:
        pass


if __name__ == '__main__':
    run()



from setuptools import setup

with open('README.rst', 'r') as file_object:
    readme = file_object.read()

setup(
    name = 'envassume',
    version = '1.0.4',
    description = 'Assume an AWS IAM role from AWS API credentials in environment variables',
    long_description = readme,
    license = 'MIT',
    author = 'Warren Moore',
    author_email = 'warren@wamonite.com',
    url = 'https://github.com/wamonite/envassume',
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Utilities',
    ],
    packages = ['envassume'],
    entry_points = dict(console_scripts = ['envassume=envassume.main:run']),
    install_requires = [
        'boto3',
        'attrs',
    ],
    zip_safe = False,
)



# -*- coding: utf-8 -*-
"""
envassume exceptions
"""


class EnvAssumeException(Exception):
    pass


class EnvAssumeHelpException(EnvAssumeException):
    pass


class EnvAssumeArgumentException(EnvAssumeException):
    pass


class EnvAssumeMissingArnException(EnvAssumeException):
    pass


class EnvAssumeMissingCommandException(EnvAssumeException):
    pass
